<?php

namespace Drupal\outbound\Plugin\Field\FieldFormatter;

use Drupal\link\Plugin\Field\FieldFormatter\LinkFormatter;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\node\NodeInterface;

/**
 * Plugin implementation of the 'outbound_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "outbound_formatter",
 *   label = @Translation("Outbound"),
 *   field_types = {
 *     "link"
 *   }
 * )
 */
class OutboundFormatter extends LinkFormatter {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $entity = $items->getEntity();

    foreach ($items as $delta => $item) {
      // By default use the full URL as the link text.
      $link_url = $this->buildUrl($item);
      $link_title = $link_url->toString();
      $node_url_outbound = '';

      // If the link text field value is available, use it for the text.
      if (!empty($item->title)) {
        // Unsanitized token replacement here because the entire link title
        // gets auto-escaped during link generation in
        // \Drupal\Core\Utility\LinkGenerator::generate().
        $link_title = \Drupal::token()->replace($item->title, [$entity->getEntityTypeId() => $entity], ['clear' => TRUE]);
      }

      // Get Current path without alias.
      $node = \Drupal::routeMatch()->getParameter('node');

      if ($node instanceof NodeInterface) {
        // You can get nid and anything else you need from the node object.
        $node_id = $node->id();
        $node_url_outbound = '/node/' . $node_id . '/outbound';
      }

      $elements[$delta] = [
        '#theme' => 'outbound_formatter',
        '#url' => $link_url,
        '#link_title' => $link_title,
        '#node_url_outbound' => $node_url_outbound,
      ];
    }

    return $elements;
  }

}
